// ****************************************************************************
// Copyright (C) 1998 by Dieter Windau
// All rights reserved
//
// colpick.h:    header file
// Version:      1.6
// Date:         08.11.1998
// Author:       Dieter Windau
//
// Freeware OWL classes that extents the dockable and gadget system
//
// TColorPicker is based on the JPColorPicker class written by Jo Parrello
//
// You are free to use/modify this code but leave this header intact.
// May not be sold for profit.
//
// Tested with Borland C++ 5.02, OWL 5.02, OWL6 patch #3 and with Windows
// NT 4.0 SP3 but I think the class should work with Windows 95 too.
// This file is provided "as is" with no expressed or implied warranty.
// Use at your own risk.
//
// This package contains many classes and source that are based on other OWL
// developers work. Very special thanks to Alan Chambers, Christopher Kohlhoff,
// Jo Parrello, Mark Hatsell, Michael Mogensen and Yura Bidus
//
// Please send me bug reports, bug fixes, enhancements, requests, etc., and
// I'll try to keep this in next versions:
//   EMail: dieter.windau@usa.net
//   Web:   http://members.aol.com/softengage/index.htm
// ****************************************************************************
#ifndef __OWLEXT_COLPICK_H
#define __OWLEXT_COLPICK_H

// Grab the core OWLEXT file we need
//
#if !defined (__OWLEXT_CORE_H)
  #include <owlext/core.h>
#endif

#include <owlext/colpick.rh>

OWLEXT_BEGIN_NAMESPACE

// Generic definitions/compiler options (eg. alignment) preceeding the
// definition of classes
//
#include <owl/preclass.h>

// ****************** Forward declaration *************************************
class OWLEXTCLASS TColorPickerData;
class OWLEXTCLASS TColorPicker;

// ******************** TColorPickerData **************************************

class OWLEXTCLASS TColorPickerData {
  public:
    static COLORREF ColorTable8[8];
    static COLORREF ColorTable16[16];
    static COLORREF ColorTable40[40];

  public:
    TColor*	Colors;
    uint		NumColors;
    uint		NumColumn;
    bool		UsePalette;
    bool		AutoParentNotify;
    bool		ShowDefaultField;
    uint		DefaultFieldResId; // string resid for the field
                                  // +1 for statusbar message
                                  // +2 for tooltip (if not present use +0)
		COLORREF DefaultFieldColor; // need if ShowDefaultField true

    bool		ShowCustomField;
    uint		CustomFieldResId;  // string resid for the field
                                  // +1 for statusbar message
                                  // +2 for tooltip (if not present use +0)
  public:
    TColorPickerData(
      COLORREF colors[],
      uint     numColors,
      uint     numColumn = 8,
      bool     usePalette = false,
		  bool     autoParentNotify = false,
      bool     showDefaultField = true,
      uint     defaultFieldResId = IDS_AUTOMATIC,
      COLORREF defaultFieldColor = 0xFF000000L,  // Color is black, first 0xFF
                                                 // identifies user defined
      bool     showCustomField = true,
      uint     customFieldResId = IDS_MORECOLORS);

    TColorPickerData(const TColorPickerData& data)
      { *this = data; }

    ~TColorPickerData();

    TColorPickerData& operator =(const TColorPickerData& d);

	  bool operator ==(const TColorPickerData& d) const;
	  bool operator !=(const TColorPickerData& d) const;
};

// ******************** TColorPicker ******************************************

#define TColorPickerMessage "TCOLORPICKERMESSAGE"

const NumCustColors = 16;
const NumMapColors  = 43; // Num of colors where a resorce Id is mapped
const BoxSize       = 18; // Size in pixel of a color rectangle
const DefaultHeight = 22; // Height of the first field
const DistToColor   = 4;  // Distance in pixel between first field and colors
const DistToCustom  = 8;  // Distance in pixel between colors and custom field
const CustomHeight  = 20; // Height of the custom field

typedef struct {
  COLORREF Color;
  uint     ResId;
} ColorTableEntry;

class OWLEXTCLASS TColorPicker : public TWindow {
  public:
    TColorPicker(TWindow* parent,
                 const TColorPickerData& data,
                 TColor   startColor,
                 int      refId,
                 LPCTSTR  title = 0,
                 TModule* module = 0);
    virtual ~TColorPicker();

  public:
#if (OWLInternalVersion >= 0x06000000L)
    virtual LPCTSTR GetClassName();
#else
    virtual char far* GetClassName();
#endif

    virtual void GetWindowClass(WNDCLASS& wndClass);
    virtual void Paint(TDC& dc, bool erase, TRect& rect);
    virtual void SetupWindow();
    virtual void ShowPickerWindow(TPoint& pt, TRect& rect);
    virtual bool PreProcessMsg(MSG& msg);
    virtual TColor GetCurrentChosenColor();
    virtual void SetCurrentChosenColor(TColor color);
    virtual void NotifyAtParent();

  public:
    static TColor GetCustomColorsValue(int index);
    static void SetCustomColorsValue(int index, TColor color);

  protected:
    void   ExitPicker();
    void   SetStatusBarMessage(int id);
    void   PaintBorder(TDC& dc, int index);
    int    GetIndexFromPoint(TPoint point);
    int    GetIndexByColor(TColor color);
    int    GetFlatState(int index);
    void   GetCellRect(int index, TRect& rect);
    void   DrawCell(TDC& dc, int index);
    void   ChosenColorChanged(int index);
    void   ColorSelectionChanged(int index);
    void   EvActivateApp(bool active, HTASK hTask);
    void   EvKillFocus(THandle hWndGetFocus );
    void   EvMouseMove(uint modKeys, TPoint& point);
    void   EvLButtonDown(uint modKeys, TPoint& point);
    void   EvPaletteChanged(THandle hWndPalChg);
    bool   EvQueryNewPalette();
    void   EvKeyDown(uint key, uint repeatCount, uint flags);

  protected:
    static COLORREF        CustColors[NumCustColors];
    static ColorTableEntry MapColorTable[NumMapColors];
    static uint            ColorPickerMessage;

  protected:
    TColorPickerData Data;
    TTooltip*        PickerTooltip;
    int              RefId;
    bool             ColorDialogOpened;
    bool             PaletteRealized;
    TColor           CurrentChosenColor;
    TColor           CurrentCustomColor;
    int              CurrentSel;
    int              ChosenColorSel;
    int              NumRows;
    int              BoxMargin;
    TSize            WindowSize;
    owl_string           DefaultText;
    owl_string           CustomText;
    LOGPALETTE*      MyLogPalette;

	DECLARE_RESPONSE_TABLE(TColorPicker);
};

// Generic definitions/compiler options (eg. alignment) following the
// definition of classes
#include <owl/posclass.h>
OWLEXT_END_NAMESPACE

#endif
