//------------------------------------------------------------------------------
// OWL Extensions (OWLEXT) Class Library
// CALCEDIT.CPP
//
// TCalcEdit Class
//
// Original code by Steve Carr (Compuserve: 100251,1571)
// Written: 13th November 1995
//
//$Revision: 1.1.2.1 $
//$Author: jogybl $
//$Date: 2007/08/23 08:17:53 $
//------------------------------------------------------------------------------
#include <owlext\pch.h>
#pragma hdrstop

#include <owl\validate.h>
#include <owl\dialog.h>

#include <owlext/calcedit.h>
#include <owlext/calcedit.rh>

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

OWLEXT_BEGIN_NAMESPACE

__OWL_USING_NAMESPACE

// private function
TModule* FindResourceModule(TWindow* parent, TModule* module, TResId resId, LPCTSTR type);


class TCalcValidator : public TFilterValidator {
	protected:
		double min;
		double max;
		TCalcEdit* owner;

	public:
		TCalcValidator( double aMin, double aMax, TCalcEdit* anOwner ) :
			TFilterValidator("0-9+-.")
		{
			min = aMin;
			max = aMax;
			owner = anOwner;
		}

	virtual void Error(TWindow*p=NULL) ;

	virtual bool IsValid( const char far* s) ;

	void SetMin( double aMin )
	{
		min = aMin ;
		if( max < min )
			max = min ;
	}
	void SetMax( double aMax )
	{
		max = aMax ;
		if( min > max )
			min = max ;
	}
	double GetMin() const { return min ; }
	double GetMax() const { return max ; }
};


void TCalcValidator::Error(TWindow*)
{
	_TCHAR buf[64] ;
	sprintf( buf, "Value must be between %.*f and %.*f",       //????????????????????
		owner->GetDecimals(), min, owner->GetDecimals(), max ) ;
	owner->MessageBox( buf, "Error", MB_ICONEXCLAMATION|MB_OK|MB_TASKMODAL);
}


bool TCalcValidator::IsValid( const char far* s )
{
	if(TFilterValidator::IsValid(s)){
		double val = atof((const char*)s) ;
		if(min == max)
			return true;
		if( val >= min && val <= max )
			return true;
	};
	return false;
}


#define CALC_DIGITS 15

class TCalc : public TDialog{
  public:
	  int  	Execute();
		void  Destroy(int retValue = IDCANCEL);
		void  CloseWindow(int retValue = IDCANCEL) ;

  public:
	 enum TCalcState { CS_FIRST, CS_VALID, CS_ERROR } ; // calculator state
	 TCalcState CalcStatus;
	 _TCHAR     Num[CALC_DIGITS + 1];
	 _TCHAR     Operator;
	 bool       Negative;
	 double     Operand;
	 _TCHAR 		LastKey;
	 TStatic*	  display;

  public:
	 TCalc( TWindow* aParent);
	 void FlashKey(_TCHAR key);
	 void Error()
	 	{
			CalcStatus = CS_ERROR;
			_tcscpy(Num, _T("Error"));
			Negative = FALSE;
	 	}

	 void SetDisplay(double r);
	 double GetDisplay() ;
	 virtual void  UpdateDisplay();

	 void CheckFirst()
	 	{
		  if (CalcStatus == CS_FIRST){
			 CalcStatus = CS_VALID;
			 _tcscpy(Num, _T("0"));
			 Negative = false;
		  }
	 	}
	 void InsertKey(char key);
	 void CalcKey(char key);
	 void Clear()
	 	{
			CalcStatus = CS_FIRST;
	   	_tcscpy(Num, _T("0"));
	   	Negative = false;
	   	Operator = _T('=');
	    LastKey = _T(' ');
	 }

  protected:
	 void			SetupWindow();

	 //
	 // override EvCommand() defined by class TWindow
	 //
	 LRESULT	EvCommand(UINT, HWND, UINT);

    // Message response functions
    //
    HBRUSH        EvCtlColor(HDC, HWND hWndChild, uint ctlType);

	 DECLARE_RESPONSE_TABLE(TCalc);
};

DEFINE_RESPONSE_TABLE1(TCalc, TDialog)
  EV_WM_CTLCOLOR,
END_RESPONSE_TABLE;
TCalc::TCalc( TWindow* parent )
:
	TDialog(parent, IDD_CALCDIALOG,
          FindResourceModule(parent,0,IDD_CALCDIALOG,RT_DIALOG))
{
	Clear();
	TWindow::Attr.AccelTable = IDA_CALCULATOR;
	display = new TStatic( this, ID_DISPLAY);
}

void TCalc::SetupWindow()
{
	TDialog::SetupWindow();

	TRect rc, rcThis = GetWindowRect(), rcParent = Parent->GetWindowRect();

	rcThis.right = rcParent.left + rcThis.Width();
	rcThis.bottom = rcParent.bottom + rcThis.Height();
	rcThis.left = rcParent.left;
	rcThis.top = rcParent.bottom;

	::GetWindowRect(::GetDesktopWindow(), &rc);
	if (rcThis.left < rc.left)
		rcThis.Offset(rc.left-rcThis.left, 0);
	if (rcThis.bottom > rc.bottom)
		rcThis.Offset(0, rc.bottom-rcThis.bottom);
	if (rcThis.right > rc.right)
		rcThis.Offset(rc.right-rcThis.right, 0);
	MoveWindow(rcThis);
}

int TCalc::Execute()
{
	if (Create()){
		if (!(TWindow::Attr.Style & WS_VISIBLE))
			ShowWindow(SW_SHOW);
		UpdateDisplay();
		return GetApplication()->BeginModal(this,MB_TASKMODAL);
	}
	return -1;
}

void TCalc::Destroy(int retval)
{
	GetApplication()->EndModal(retval);
	TWindow::Destroy(retval);
}

void TCalc::CloseWindow(int retval)
{
	if (CanClose()){
		TransferData(tdGetData);
		Destroy(retval);
	}
}

LRESULT TCalc::EvCommand(UINT id, HWND hWndCtl, UINT notifyCode)
{
  if (hWndCtl != 0 && notifyCode == BN_CLICKED)
	 CalcKey( _TCHAR(id) );
  else if ( hWndCtl == 0 && notifyCode == 1 ){
  	FlashKey(_TCHAR(id));
	 	CalcKey(_TCHAR(id));
  }
  return TDialog::EvCommand(id, hWndCtl, notifyCode) ;
}

//
// Colorize the calculator. Allows background to show through corners of
// buttons, uses yellow text on black background in the display, and sets
// the dialog background to blue.
//
HBRUSH
TCalc::EvCtlColor(HDC hDC, HWND hWndChild, uint ctlType)
{
  switch (ctlType) {
    case CTLCOLOR_STATIC:
      SetTextColor(hDC, TColor::LtYellow);
      SetBkColor(hDC, TColor::Black);
      //SetBkMode(hDC, TRANSPARENT);
      return (HBRUSH)GetStockObject(BLACK_BRUSH);
    default:
      return TDialog::EvCtlColor(hDC, hWndChild, ctlType);
  }
}

void
TCalc::UpdateDisplay()
{
  _TCHAR  str[CALC_DIGITS + 2] ;

  if (Negative)
	  _tcscpy(str, _T("-"));
  else
	  str[0] = _T('\0');
  _tcscat( str, Num ) ;

  display->SetText( str ) ;
}

void TCalc::SetDisplay(double r)
{
  _TCHAR* first;
  _TCHAR* last;
  int    	charsToCopy;
  _TCHAR 	str[64];

  r = (floor(r * 10000000L + .5)) / 10000000L;

  sprintf(str, "%0.10f", r);
  first = str;
  Negative = FALSE;

  if(str[0] == '-'){
	 first++;
	 Negative = true;
  }

  if (_tcslen(first) > CALC_DIGITS + 1 + 10 )
    Error();
  else{
	 last = strchr(first, 0);

	 while (last[-1] == _T('0'))
		--last;

	 if (last[-1] == _T('.'))
		--last;

	 charsToCopy = min(CALC_DIGITS + 1, int(last - first));
	 strncpy(Num, first, charsToCopy);
	 Num[charsToCopy] = 0;
  }
}


void TCalc::FlashKey(char key)
{
  if (key == '\n')
  	key = '=' ;

  HWND button = GetDlgItem( toupper(key) ) ;
  if (button){
  	::SendMessage(button, BM_SETSTATE, 1, 0) ;
#ifdef __WIN32__
	 	Sleep(100);
#else
	 	for (int zz = 1; zz <= 30000 ; ++zz)
   		;
#endif
	 	::SendMessage(button, BM_SETSTATE, 0, 0) ;
  }
}

double
TCalc::GetDisplay()
{
  double r = atof(Num);

  if (Negative)
	 r = -r;

  return r ;
}

void
TCalc::InsertKey(char key)
{
  int l = strlen(Num);

  if (l < CALC_DIGITS){
	 Num[l++] = key;
	 Num[l] = '\0';
  }
}

void
TCalc::CalcKey(char key)
{
  key = (char)toupper(key);

  if (CalcStatus == CS_ERROR && key != 'C')
	 key = ' ';

  if (key >= '0' && key <= '9'){
		CheckFirst();

  	if (!strcmp(Num, "0"))
			Num[0] = '\0';

		InsertKey(key);
	}
  else if (key == '+' || key == '-' || key == '*' ||
			key == '/' || key == '=' || key == '%' || key == 0x0D)
	{
	 	if( ( key == 0x0D || key == '=' ) && key == LastKey )
			CloseWindow( IDOK ) ;

	 	if (CalcStatus == CS_VALID){
			CalcStatus = CS_FIRST;
			double  r = GetDisplay() ;

			if (key == '%'){
		  	switch(Operator){
			 		case '+':
			 		case '-':
						r = Operand * r / 100;
					  break;

			 		case '*':
			 		case '/':
						r /= 100;
						break;
		  	}
			}

			switch(Operator){
		  	case '+':
			 		SetDisplay(Operand + r);
			 		break;

		  	case '-':
			 		SetDisplay(Operand - r);
			 		break;

		  	case '*':
			 		SetDisplay(Operand * r);
			 		break;

		  	case '/':
			 		if (r == 0)
						Error();
			 		else
						SetDisplay(Operand / r);
					 break;
      }
	 	}

	 	Operator = key;
	 	Operand = GetDisplay();

	}
  else{
  	switch(key){
			case '.':
		  	CheckFirst();
		  	if (!strchr(Num, '.'))
			 		InsertKey(key);
		  	break;

			case 0x8:
		  	CheckFirst();
		  	if (strlen(Num) == 1)
				 	strcpy(Num, "0");
		  	else
			 		Num[strlen(Num) - 1] = '\0';
		  	break;

			case '_':
		  	Negative = !Negative;
		  	break;

			case 'C':
		  	Clear();
		  	break;
    }
  }
  LastKey = key ;

  UpdateDisplay();
}


TCalcEdit::TCalcEdit(TWindow* parent, int resourceId,
							uint aWidth, uint aDecimals, double aMin, double aMax,
							TModule* module)
:
	TPopupEdit(parent, resourceId, IDB_CALCBTN, aWidth+1, module)
{
//	SetText("0"); //?????????????????
	SetCaption("0");
	aCalc = NULL;
	decimals = aDecimals ;
	Validator = new TCalcValidator( aMin, aMax, this ) ;
}

TCalcEdit::TCalcEdit(TWindow* parent, int id, double aVal, int x, int y,
				  int w, int h, uint aWidth, uint aDecimals, double aMin, double aMax,
				  TModule* module )
:
	TPopupEdit( parent, id, "",  x, y, w, h, IDB_CALCBTN, aWidth+1, module )
{
	aCalc = NULL ;
	decimals = aDecimals ;
	Validator = new TCalcValidator( aMin, aMax, this ) ;
	operator =( aVal ) ;
}


void TCalcEdit::Clicked()
{
	TCalc* pCalc = new TCalc(this) ;
	((TWindow*)pCalc)->Attr.AccelTable = IDA_CALCULATOR;

	GetText( pCalc->Num, CALC_DIGITS );
	double r = atof( pCalc->Num );
	pCalc->CalcStatus = TCalc::CS_VALID;
	pCalc->SetDisplay( r );

	if( pCalc->Execute() == IDOK )
		operator =(pCalc->GetDisplay());

	delete pCalc;
}

void TCalcEdit::SetMin( double d )
{
	TCalcValidator *cv = (TCalcValidator*)Validator ;
	if( cv )
		cv->SetMin( d ) ;
}

void TCalcEdit::SetMax( double d )
{
	TCalcValidator *cv = (TCalcValidator*)Validator ;
	if( cv )
		cv->SetMax( d ) ;
}

void TCalcEdit::operator = ( double d )
{
	_TCHAR buf[64];
	sprintf( buf, "%.*f", decimals, d );
	SetText(buf);
}

TCalcEdit::operator double()
{
	_TCHAR buf[64] ;
	GetText( buf, sizeof(buf) ) ;
	return atof(buf) ;
}

OWLEXT_END_NAMESPACE
//==============================================================================
