//-------------------------------------------------------------------
// OWL Extensions (OWLEXT) Class Library
// Copyright(c) 1996 by Manic Software.
// All rights reserved.
//
// This file implements the TEnumWindows class.  It is encapsulates the
// EnumTaskWindows (16-bit) or EnumThreadWindows (32-bit) APIs.  It can
// also handle EnumChildWindows to visit every window for a task/thread.
//
// Original code by Don Griffin; used with permission.
//$Revision: 1.1.2.1 $
//$Author: jogybl $
//$Date: 2007/08/23 08:17:54 $
//-------------------------------------------------------------------
#include <owlext\pch.h>
#pragma hdrstop

#include <owlext/EnumWnd.h>


OWLEXT_BEGIN_NAMESPACE


TEnumWindows::TEnumWindows ()
{
	mDoingChildWindows = false;
}


void TEnumWindows::EnumWindows (TEnumMethod em)
{
	LPARAM  lParam = reinterpret_cast<LPARAM> (this);

  mDoingChildWindows = (em == emTopLevelWindows);

#if defined(BI_PLAT_WIN32)
	EnumThreadWindows (GetCurrentThreadId(), WndEnumProc, lParam);
#else
	EnumTaskWindows (GetCurrentTask(), WndEnumProc, lParam);
#endif
}


bool TEnumWindows::WndEnumMethod (HWND hWnd)
{
	if (!OnEnumWindow (hWnd))
  	return false;

  if (! mDoingChildWindows){
  	mDoingChildWindows = true;
    ::EnumChildWindows (hWnd, WndEnumProc, reinterpret_cast<LPARAM>(this));
    mDoingChildWindows = false;
  }

  return true;
}

BOOL CALLBACK IF16(__export)
TEnumWindows::WndEnumProc (HWND hWnd, LPARAM lParam)
{
	TEnumWindows  * object = reinterpret_cast <TEnumWindows *> (lParam);
  return object->WndEnumMethod (hWnd);
}


OWLEXT_END_NAMESPACE

