# This is a part of the ObjectWindows library.
# Copyright (C) 1997 - 1998 Yura Bidus
# All rights reserved.
#
# OWLX3[D]V.DLL is a DLL
#  which exports all the OCF classes
#
# If you need a private build of the OCF DLL, be sure to rename
#  "OWLX3.DLL" to something more appropriate for your application.
# Please do not re-distribute a privately built version with the
#  name "OWLX3.DLL".
#
# Use nmake /f msocfdll.mak LIBNAME=<your name> OUTDIR=outdir to do this.
#
# Note: LIBNAME must be 6 characters or less.

!ifndef LIBNAME
!error LIBNAME is not defined. LIBNAME=OWL60 builds the prebuilt DLL.
!endif

# additional options
#TARGOPTS =/P

!ifndef OUTDIR
OUTDIR=..\..\bin
!endif

!ifndef REGEN
REGEN=0
!endif

# Default to DEBUG mode
!ifndef DEBUG
DEBUG=1
!endif

# Default Codeview Info
!ifndef CODEVIEW
!if "$(DEBUG)" == "1"
CODEVIEW=1
!else
CODEVIEW=0
!endif
!endif

!if "$(DEBUG)" != "0"
CRTDLL=msvcrtd.lib msvcirtd.lib ../../lib/owlvdi.lib
!if "$(BI_STD_RTL)" == "1"
CRTDLL=$(CRTDLL) msvcprtd.lib
!else
CRTDLL=$(CRTDLL) msvcirtd.lib
!endif
!else
CRTDLL=msvcrt.lib msvcirt.lib ../../lib/owlvi.lib
!if "$(BI_STD_RTL)" == "1"
CRTDLL=$(CRTDLL) msvcprt.lib
!else
CRTDLL=$(CRTDLL) msvcirt.lib
!endif
!endif

TARGET=w
DLL=2
#TARGDEFS=/D_OWL_CORE_IMPL
LFLAGS=/nodefaultlib
RCFLAGS=$(RCFLAGS) /r

!if "$(UNICODE)" == "1"
TARGTYPE=u
RCDEFINES=$(RCDEFINES) /D_UNICODE
!else
!if "$(MSCVER)" != "5"
LFLAGS=$(LFLAGS) /filealign:4096
!endif
!endif

!if "$(DEBUG)" != "0"
# Debug DLL build
TARGTYPE=$(TARGTYPE)d
RCDEFINES=$(RCDEFINES) /D_DEBUG
LFLAGS=$(LFLAGS)
!ifndef MONOLITHIC
MONOLITHIC=0
!endif
!else
# Release DLL build
RCDEFINES=$(RCDEFINES)
LFLAGS=$(LFLAGS)
!ifndef MONOLITHIC
MONOLITHIC=1
!endif
!endif

!if "$(MONOLITHIC)" == "1"
TARGDEFS=$(TARGDEFS)
RCDEFINES=$(RCDEFINES)
!endif

CFNAME=$(LIBNAME)$(TARGTYPE)
TARG=$(LIBNAME)$(OWLVER)v$(PF)$(TARGTYPE)
TARG_STATIC=$(LIBNAME)v$(PF)$(TARGTYPE)i

# OPT:noref keeps unreferenced functions (ie. no dead-code elimination)
!if "$(REGEN)" == "0"
#LFLAGS=$(LFLAGS) /opt:ref
LFLAGS=$(LFLAGS) /opt:ref /opt:icf,32
!else
LFLAGS=$(LFLAGS) /opt:noref
!endif

DEFFILE=$(PLATFORM)\$(TARG).DEF

!if "$(DEBUGTYPE)" == ""
#DEBUGTYPE=cv,fixup
DEBUGTYPE=cv
!endif

!if "$(CODEVIEW)" != "0"
!if "$(REGEN)" != "1"
LFLAGS=$(LFLAGS) /debug:full /debugtype:$(DEBUGTYPE)
!else
LFLAGS=$(LFLAGS) /debug:none
!endif
!if "$(NO_PDB)" != "1" && "$(REGEN)" != "1"
LFLAGS=$(LFLAGS) /pdb:$(OUTDIR)\$(TARG).pdb
!else
LFLAGS=$(LFLAGS) /pdb:none
!endif
!else
LFLAGS=$(LFLAGS) /debug:none
!if "$(INCREMENTAL)" != "1"
LFLAGS=$(LFLAGS) /incremental:no
!endif
!endif

!ifdef RELEASE # Release VERSION info
RCDEFINES=$(RCDEFINES) /DRELEASE
LFLAGS=$(LFLAGS) /release
!endif

LFLAGS=$(LFLAGS) /dll

!if "$(ORDER)" == "1"
!if exist($(PLATFORM)\$(TARG).prf)
DEFS=$(DEFS) /D_OCF_FUNCTION_ORDER
LFLAGS=$(LFLAGS) /order:@$(PLATFORM)\$(TARG).prf
!endif
!endif


dll_goal: create2.dir \
        $(OUTDIR)\$(TARG).dll #..\..\lib\$(TARG_STATIC).lib

################################################################################
# import most rules and library files from normal makefile

!include mscMake

create2.dir:
	@-if not exist $D\*.* mkdir $D

################################################################################
# more flags and switches

LFLAGS=$(LFLAGS) /version:3.00
!if "$(UNICODE)" == "1"
LFLAGS=$(LFLAGS) /base:0x4F600000
!else
LFLAGS=$(LFLAGS) /base:0x4F200000
!endif
LIBS=$(CRTDLL) kernel32.lib gdi32.lib user32.lib uuid.lib \
shell32.lib advapi32.lib comdlg32.lib \
ole32.lib oleaut32.lib $(PROFLIB)

#############################################################################
# Build target

$D\$(TARG).res: owlext.rc
        rc $(RCFLAGS) $(RCDEFINES) /fo $D\$(TARG).res owlext.rc

DLL_OBJS=$(OBJECT) $(OBJS_DLL) $(OWLASM) #$(NO_HDR_OBJ)

!if "$(MONOLITHIC)" == "1"
DLL_OBJS=$(DLL_OBJS) 
!endif

#DLL_OBJS=$(DLL_OBJS)

DLL_RESOURCES=$D\$(TARG).res


$(OUTDIR)\$(TARG).dll: $(DLL_OBJS) $(DEFFILE) $(DLL_RESOURCES)
	link @<<
$(LFLAGS)
$(LIBS)
$(DLL_OBJS)
$(DLL_RESOURCES)
/def:$(DEFFILE)
/out:$(OUTDIR)\$(TARG).dll
/pdb:$(OUTDIR)\$(TARG).pdb
/map:$D\$(TARG).MAP
/implib:..\..\lib\$(TARG_STATIC).lib
!if "$(MSCVER)" != "5"
/ignore:4037
/ignore:4065
/ignore:4199
/filealign:4096
!endif
<<

#############################################################################
